#!perl -T

use Test::More tests => 1;

BEGIN {
	use_ok( 'HTML::WikiConverter::GoogleCode' );
}

diag( "Testing HTML::WikiConverter::GoogleCode $HTML::WikiConverter::GoogleCode::VERSION, Perl $], $^X" );
